# phaxelf-lib
The **Pha**[ser] [Rising Pi]**x**[el] **Elf** [Games Works] **lib**rary.

*Go phax your elf.*

Online documentation at [http://risingpixel.github.io/phaxelf-lib](http://risingpixel.github.io/phaxelf-lib)

## What is this?

A random set of utility classes to build better Phaser games.

## Installation
To install this library in your project:

```bash
npm install --save phaxelf-lib
```

Or with yarn:

```bash
yarn add phaxelf-lib
```

## Example Usage

### Quickstart
After installing the library, you can just require what you need from the package:

```javascript
import { Drag } from 'phaxelf-lib';

mySprite = game.add.sprite(0, 0, 'somesprite.png');
Drag.enable(mySprite);
```

### Extensions
Phaxelf provides a set of extensions for the default phaser methods, to handle scaling, coordinates and tweening easier.

To use them, you have to initialize it manually in your game startup.

```javascript
import { Extensions } from 'phaxelf-lib';

// ... your main file
Extensions.init(LU);
/// ... other stuff
```

Then, you can just find them ready as methods.

#### Moving a sprite in the hierarchy in the same screen position of another one

```javascript
mySpriteInAScaledGroup = game.add.sprite(0, 0, 'sprite1.png');
mySpriteInAnotherMessyScaledGroup = game.add.sprite(0, 0, 'sprite1.png');

mySpriteInAScaledGroup.$$moveTo(mySpriteInAnotherMessyScaledGroup);
```

#### Check if a point is contained in a sprite, no physics

```javascript
sprite.$$contains(new Phaser.Point(100, 100)); // => true/false
```

#### Shortcut for tweens

```javascript
let tween = sprite.$$tween({ x: 100, y: 100}, 1000, { repeat: 10, yoyo: true, autostart: true });
```

#### Tint tweening

```javascript
let tween = sprite.$$tintTween(0xff0000, 1000);
```

#### Scale tweening

```javascript
let tween = sprite.$$scaleTween({ x: '+0.5', y: '+0.5' }, 1000, { yoyo: true });
```

Complete reference [here](http://risingpixel.github.io/phaxelf-lib).

### Library Local Development
To test this library locally, first of all install the npm dependencies. Remember to run it every time you update them:

```bash
npm install
```

Build the library by using gulp:

```bash
npm run start
```

It will automatically rebuild the sources when you change the files.

Then, to add it in your project, specifying the current folder where you've downloaded the library sources.

```bash
npm install --save path/to/phaxelf-lib # Replace this with your library path
```

You may need to re-run this two last commands during the development cycle to update the sources in your project.

### Pro tip
You can concatenate this commands to make the development cycle easier, like:

```bash
npm install --save path/to/phaxelf-lib && npm run start:dev
```

On windows:

```batch
npm install --save path\to\phaxelf-lib; npm run start:dev
```

## Docs generation and publishing
Docs are auto-generated by the gulp task.

To update the online documentation, just run:

```bash
npm run docs:publish
```

## Library publishing

* Make sure you're logged in in npm, using `npm whoami`. If not, just register using `npm adduser`.
* Update the version number in the `package.json` file, using [semantic versioning](http://semver.io). To achieve this, use `npm version patch`, replacing `patch` with one of `patch`, `minor`, `major`.
* Run `npm publish` to publish the new release
